@@ -1,6 +1,6 @@
 /************************************************************************************\
   bb_ledm.c - HP SANE backend support for ledm based multi-function peripherals
-  (c) 2010 Copyright Hewlett-Packard Development Company, LP
+  (c) 2010 Copyright HP Development Company, LP
 
   Primary Author: Naga Samrat Chowdary, Narla
   Contributing Authors: Yashwant Kumar Sahu, Sarbeswar Meher
@@ -197,7 +197,7 @@ Keep-Alive: 20\r\nProxy-Connection: keep-alive\r\nCookie: AccessCounter=new\r\n0
 <GrayRendering>NTSC</GrayRendering>\
 <ToneMap>\
 <Gamma>0</Gamma>\
-<Brightness>1000</Brightness>\
+<Brightness>%d</Brightness>\
 <Contrast>%d</Contrast>\
 <Highlite>0</Highlite>\
 <Shadow>0</Shadow></ToneMap>\
@@ -232,6 +232,7 @@ Keep-Alive: 300\r\nProxy-Connection: keep-alive\r\nCookie: AccessCounter=new\r\n
 # define JOBSTATE_PROCESSING "<j:JobState>Processing</j:JobState>"
 # define JOBSTATE_CANCELED "<j:JobState>Canceled</j:JobState>"
 # define JOBSTATE_COMPLETED "<j:JobState>Completed</j:JobState>"
+# define PRESCANPAGE "<PreScanPage>"
 
 static int parse_scan_elements(const char *payload, int size, struct wscn_scan_elements *elements)
 {
@@ -690,17 +691,26 @@ int bb_open(struct ledm_session *ps)
   ps->adf_tlyRange.max = SANE_FIX(pbb->elements.config.adf.maximum_size.height/11.811023);
   ps->adf_bryRange.max = ps->adf_tlyRange.max;
 
-  i = pbb->elements.config.platen.platen_resolution_list[0] + 1;
-  while(i--)
+  if (pbb->elements.config.platen.flatbed_supported)
   {
-  _DBG("bb_open platen_resolution_list = %d\n",  pbb->elements.config.platen.platen_resolution_list[i]);
-    ps->platen_resolutionList[i] = pbb->elements.config.platen.platen_resolution_list[i];
-    ps->resolutionList[i] = pbb->elements.config.platen.platen_resolution_list[i];
+      i = pbb->elements.config.platen.platen_resolution_list[0] + 1;
+      while(i--)
+      {
+          _DBG("bb_open platen_resolution_list = %d\n",  pbb->elements.config.platen.platen_resolution_list[i]);
+          ps->platen_resolutionList[i] = pbb->elements.config.platen.platen_resolution_list[i];
+          ps->resolutionList[i] = pbb->elements.config.platen.platen_resolution_list[i];
+      }
+  }
+  if (pbb->elements.config.adf.supported)
+  {
+     i = pbb->elements.config.adf.adf_resolution_list[0] + 1;
+     while(i--)
+     {
+         _DBG("bb_open adf_resolution_list = %d\n", pbb->elements.config.adf.adf_resolution_list[i]);
+         ps->adf_resolutionList[i] = pbb->elements.config.adf.adf_resolution_list[i]; 
+         ps->resolutionList[i] = pbb->elements.config.adf.adf_resolution_list[i];
+     }
   }
-
-  i = pbb->elements.config.adf.adf_resolution_list[0] + 1;
-  while(i--) ps->adf_resolutionList[i] = pbb->elements.config.adf.adf_resolution_list[i]; 
-
   stat = 0;
 
 bugout:
@@ -709,9 +719,11 @@ bugout:
 
 int bb_close(struct ledm_session *ps)
 {
-  _DBG("bb_close()\n");
-  free(ps->bb_session);
-  ps->bb_session = NULL;
+  if (ps->bb_session)
+  {
+      free(ps->bb_session);
+      ps->bb_session = NULL;
+  }
   return 0;
 } 
 
@@ -801,10 +813,12 @@ int bb_is_paper_in_adf(struct ledm_session *ps) /* 0 = no paper in adf, 1 = pape
 
   if(http_open(ps->dd, HPMUD_S_LEDM_SCAN, &pbb->http_handle) != HTTP_R_OK)
   {
+        _BUG("unable to open channel HPMUD_S_LEDM_SCAN \n");
+        return -1;
   }
   if (http_write(pbb->http_handle, GET_SCANNER_STATUS, sizeof(GET_SCANNER_STATUS)-1, 10) != HTTP_R_OK)
   {
-    //goto bugout;
+    _BUG("unable to get scanner status \n");
   }
   read_http_payload(ps, buf, sizeof(buf), EXCEPTION_TIMEOUT, &bytes_read);
 
@@ -828,7 +842,7 @@ SANE_Status bb_start_scan(struct ledm_session *ps)
 {
   char buf[4096] = {0};
   char buf1[1024]={0};
-  int len, bytes_read;
+  int len, bytes_read, paper_status;
   int i, timeout = 10 ;
   char szPage_ID[5] = {0};
   char szJob_ID[5] = {0};
@@ -841,12 +855,14 @@ SANE_Status bb_start_scan(struct ledm_session *ps)
   {
     if(http_open(ps->dd, HPMUD_S_LEDM_SCAN, &pbb->http_handle) != HTTP_R_OK)
     {
-       // goto bugout;
+        _BUG("unable to open channel HPMUD_S_LEDM_SCAN \n");
+        goto bugout;
     }
 
     if (http_write(pbb->http_handle, GET_SCANNER_STATUS, sizeof(GET_SCANNER_STATUS)-1, timeout) != HTTP_R_OK)
     {
-       //goto bugout;
+       _BUG("unable to GET_SCANNER_STATUS \n");
+       goto bugout;
     }
  
     read_http_payload(ps, buf, sizeof(buf), timeout, &bytes_read);
@@ -862,21 +878,24 @@ SANE_Status bb_start_scan(struct ledm_session *ps)
 
     if(http_open(ps->dd, HPMUD_S_LEDM_SCAN, &pbb->http_handle) != HTTP_R_OK)
     {
+        _BUG("unable to open channel HPMUD_S_LEDM_SCAN \n");
+        goto bugout;
     }
 
     len = snprintf(buf, sizeof(buf), CREATE_SCAN_JOB_REQUEST,
-  		ps->currentResolution,//<XResolution>
-  		ps->currentResolution,//<YResolution>
-    	(int) (ps->currentTlx / 5548.7133),//<XStart>
-    	(int) ((ps->currentBrx / 5548.7133) - (ps->currentTlx / 5548.7133)),//<Width>
-    	(int) (ps->currentTly / 5548.7133),//<YStart>
-    	(int) ((ps->currentBry / 5548.7133) - (ps->currentTly / 5548.7133)),//<Height>
-    	"Jpeg",//<Format>
-    	(! strcmp(ce_element[ps->currentScanMode], "Color8")) ? "Color" : (! strcmp(ce_element[ps->currentScanMode], "Gray8")) ? "Gray" : "Gray",//<ColorSpace>
-    	((! strcmp(ce_element[ps->currentScanMode], "Color8")) || (! strcmp(ce_element[ps->currentScanMode], "Gray8"))) ? 8: 8,//<BitDepth>
-    	ps->currentInputSource == IS_PLATEN ? is_element[1] : is_element[2],//<InputSource>
-    	ps->currentInputSource == IS_PLATEN ? is_element[1] : is_element[2],//<InputSourceType>
-    	ps->currentInputSource != IS_ADF_DUPLEX ? "" : "<AdfOptions><AdfOption>Duplex</AdfOption></AdfOptions>",
+        ps->currentResolution,//<XResolution>
+        ps->currentResolution,//<YResolution>
+        (int) (ps->currentTlx / 5548.7133),//<XStart>
+        (int) ((ps->currentBrx / 5548.7133) - (ps->currentTlx / 5548.7133)),//<Width>
+        (int) (ps->currentTly / 5548.7133),//<YStart>
+        (int) ((ps->currentBry / 5548.7133) - (ps->currentTly / 5548.7133)),//<Height>
+        "Jpeg",//<Format>
+        (! strcmp(ce_element[ps->currentScanMode], "Color8")) ? "Color" : (! strcmp(ce_element[ps->currentScanMode], "Gray8")) ? "Gray" : "Gray",//<ColorSpace>
+        ((! strcmp(ce_element[ps->currentScanMode], "Color8")) || (! strcmp(ce_element[ps->currentScanMode], "Gray8"))) ? 8: 8,//<BitDepth>
+        ps->currentInputSource == IS_PLATEN ? is_element[1] : is_element[2],//<InputSource>
+        ps->currentInputSource == IS_PLATEN ? is_element[1] : is_element[2],//<InputSourceType>
+        ps->currentInputSource != IS_ADF_DUPLEX ? "" : "<AdfOptions><AdfOption>Duplex</AdfOption></AdfOptions>",
+        (int)ps->currentBrightness,//<Brightness>
         (int)ps->currentContrast);//<Contrast>
 
     len = len + strlen(ZERO_FOOTER);
@@ -949,6 +968,8 @@ SANE_Status bb_start_scan(struct ledm_session *ps)
 
   if(http_open(ps->dd, HPMUD_S_LEDM_SCAN, &pbb->http_handle) != HTTP_R_OK)
   {
+      _BUG("unable to open channel HPMUD_S_LEDM_SCAN \n");
+      goto bugout;
   }
   while(strstr(buf, READY_TO_UPLOAD) == NULL)
   {
@@ -957,26 +978,32 @@ SANE_Status bb_start_scan(struct ledm_session *ps)
 
      if (http_write(pbb->http_handle, buf, strlen(buf), 1) != HTTP_R_OK)
      {
-	//goto bugout;
-	break ;
+        //goto bugout;
+        break ;
      }
      if (read_http_payload (ps, buf, sizeof(buf), 5, &len) != HTTP_R_OK)
      {
         //goto bugout
         _DBG("bb_start_scan() read_http_payload FAILED len=%d buf=%s\n", len, buf);
         break;
+     }
+      //For a new scan, buf must contain <PreScanPage>. 
+     if (NULL == strstr(buf,PRESCANPAGE)) 
+     {         //i.e Paper is not present in Scanner
+         stat = SANE_STATUS_NO_DOCS;
+         goto bugout;
      }
      if (strstr(buf,JOBSTATE_CANCELED) || strstr(buf, CANCELED_BY_DEVICE) || strstr(buf, CANCELED_BY_CLIENT))
      {
-       	//_DBG("bb_start_scan() SCAN CANCELLED\n");
-       	stat = SANE_STATUS_GOOD;
-       	ps->user_cancel = 1;
-       	goto bugout;
+        _DBG("bb_start_scan() SCAN CANCELLED\n");
+        stat = SANE_STATUS_GOOD;
+        ps->user_cancel = 1;
+        goto bugout;
      }
      if (strstr(buf, JOBSTATE_COMPLETED))
      {
-	stat = SANE_STATUS_GOOD;
-	goto bugout;
+        stat = SANE_STATUS_GOOD;
+        goto bugout;
      }
      usleep(500000);//0.5 sec delay
   }//end while()
