@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2001-2009 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2001-2015 HP Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,8 +31,9 @@ import signal
 # Local
 from base.g import *
 from base import device, utils, models, pkit
+from base.sixext import  to_unicode
 from prnt import cups
-from ui_utils import load_pixmap
+from .ui_utils import load_pixmap
 from installer import pluginhandler
 
 try:
@@ -46,9 +47,9 @@ except ImportError:
 
 # Qt
 from qt import *
-from setupform_base import SetupForm_base
-from setupsettings import SetupSettings
-from setupmanualfind import SetupManualFind
+from .setupform_base import SetupForm_base
+from .setupsettings import SetupSettings
+from .setupmanualfind import SetupManualFind
 
 
 class DeviceListViewItem(QListViewItem):
@@ -69,12 +70,12 @@ class PrinterNameValidator(QValidator):
         QValidator.__init__(self, parent, name)
 
     def validate(self, input, pos):
-        input = unicode(input)
+        input = to_unicode(input)
 
         if not input:
             return QValidator.Acceptable, pos
 
-        elif input[pos-1] in u"""~`!@#$%^&*()-=+[]{}()\\/,.<>?'\";:| """:
+        elif input[pos-1] in """~`!@#$%^&*()-=+[]{}()\\/,.<>?'\";:| """:
             return QValidator.Invalid, pos
 
         # TODO: How to determine if unicode char is "printable" and acceptable
@@ -92,12 +93,12 @@ class PhoneNumValidator(QValidator):
         QValidator.__init__(self, parent, name)
 
     def validate(self, input, pos):
-        input = unicode(input)
+        input = to_unicode(input)
 
         if not input:
             return QValidator.Acceptable, pos
 
-        elif input[pos-1] not in u'0123456789-(+) ':
+        elif input[pos-1] not in '0123456789-(+) ':
             return QValidator.Invalid, pos
 
         else:
@@ -235,7 +236,7 @@ class SetupForm(SetupForm_base):
                     return
                 if not ok or pluginObj.getStatus() != pluginhandler.PLUGIN_INSTALLED:
                     if plugin == PLUGIN_REQUIRED:
-                        self.FailureUI(self.__tr("<b>The printer you are trying to setup requires a binary driver plug-in and it failed to install.</b><p>Please check your internet connection and try again.</p><p>Visit <u>http://hplipopensource.com</u> for more information.</p>"))
+                        self.FailureUI(self.__tr("<b>The device you are trying to setup requires a binary plug-in. Some functionalities may not work as expected without plug-ins.<p> Please run 'hp-plugin' as normal user to install plug-ins.</b></p><p>Visit <u>http://hplipopensource.com</u> for more infomation.</p>"))
                         return
                     else:
                         self.WarningUI(self.__tr("Either you have chosen to skip the installation of the optional plug-in or that installation has failed.  Your printer may not function at optimal performance."))
@@ -565,7 +566,7 @@ class SetupForm(SetupForm_base):
 
     def otherPPDPushButton_clicked(self):
         ppd_dir = sys_conf.get('dirs', 'ppd')
-        ppd_file = unicode(QFileDialog.getOpenFileName(ppd_dir, "PPD Files (*.ppd *.ppd.gz);;All Files (*)", self, "open file dialog", "Choose a PPD file"))
+        ppd_file = to_unicode(QFileDialog.getOpenFileName(ppd_dir, "PPD Files (*.ppd *.ppd.gz);;All Files (*)", self, "open file dialog", "Choose a PPD file"))
 
         if ppd_file and os.path.exists(ppd_file):
             self.updatePPDPage({ppd_file: cups.getPPDDescription(ppd_file)})
@@ -596,6 +597,15 @@ class SetupForm(SetupForm_base):
         # Check for duplicate names
         if (self.device_uri in self.installed_print_devices and printer_name in self.installed_print_devices[self.device_uri]) \
            or (printer_name in installed_printer_names):
+            warn_text = self.__tr("<b>One or more print queues already exist for this device: %s</b>.<br> <b>Would you like to install another print queue for this device ?</b>" %
+                    ', '.join([printer.encode('utf-8') for printer in installed_printer_names if printer_name in printer]))
+            if ( QMessageBox.warning(self,
+                                self.caption(),
+                                warn_text,
+                                QMessageBox.Yes,
+                                QMessageBox.No,
+                                QMessageBox.NoButton) == QMessageBox.Yes ):
+
                 i = 2
                 while True:
                     t = printer_name + "_%d" % i
@@ -603,6 +613,8 @@ class SetupForm(SetupForm_base):
                         printer_name += "_%d" % i
                         break
                     i += 1
+            else:
+                self.close()
 
         self.printer_name_ok = True
         self.printerNameLineEdit.setText(printer_name)
@@ -641,7 +653,7 @@ class SetupForm(SetupForm_base):
 
 
     def printerNameLineEdit_textChanged(self,a0):
-        self.printer_name = unicode(a0)
+        self.printer_name = to_unicode(a0)
         self.defaultPrinterNamePushButton.setEnabled(True)
 
         self.printer_name_ok = True
@@ -675,16 +687,16 @@ class SetupForm(SetupForm_base):
 
 
     def printerLocationLineEdit_textChanged(self, a0):
-        self.location = unicode(a0)
+        self.location = to_unicode(a0)
 
     def printerDescriptionLineEdit_textChanged(self,a0):
-        self.desc = unicode(a0)
+        self.desc = to_unicode(a0)
 
     def faxLocationLineEdit_textChanged(self,a0):
-        self.fax_location = unicode(a0)
+        self.fax_location = to_unicode(a0)
 
     def faxDescriptionLineEdit_textChanged(self,a0):
-        self.fax_desc = unicode(a0)
+        self.fax_desc = to_unicode(a0)
 
     def defaultPrinterNamePushButton_clicked(self):
         self.setDefaultPrinterName()
@@ -722,7 +734,7 @@ class SetupForm(SetupForm_base):
         #self.fax_name_error = False
 
     def faxNameLineEdit_textChanged(self, a0):
-        self.fax_name = unicode(a0)
+        self.fax_name = to_unicode(a0)
         self.defaultFaxNamePushButton.setEnabled(True)
 
         self.fax_name_ok = True
@@ -756,10 +768,10 @@ class SetupForm(SetupForm_base):
 
 
     def faxNumberLineEdit_textChanged(self, a0):
-        self.fax_number = unicode(a0)
+        self.fax_number = to_unicode(a0)
 
     def faxNameCoLineEdit_textChanged(self, a0):
-        self.fax_name_company = unicode(a0)
+        self.fax_name_company = to_unicode(a0)
 
     def faxCheckBox_clicked(self):
         pass
@@ -788,7 +800,7 @@ class SetupForm(SetupForm_base):
                     d.open()
                 except Error:
                     error_text = self.__tr("Unable to communicate with the device. Please check the device and try again.")
-                    log.error(unicode(error_text))
+                    log.error(to_unicode(error_text))
                     if QMessageBox.critical(self,
                                            self.caption(),
                                            error_text,
@@ -807,15 +819,15 @@ class SetupForm(SetupForm_base):
 
                             try:
                                 if read:
-                                    self.fax_number = unicode(d.getPhoneNum())
-                                    self.fax_name_company = unicode(d.getStationName())
+                                    self.fax_number = to_unicode(d.getPhoneNum())
+                                    self.fax_name_company = to_unicode(d.getStationName())
                                 else:
                                     d.setStationName(self.fax_name_company)
                                     d.setPhoneNum(self.fax_number)
 
                             except Error:
                                 error_text = self.__tr("<b>Device I/O Error</b><p>Could not communicate with device. Device may be busy.")
-                                log.error(unicode(error_text))
+                                log.error(to_unicode(error_text))
 
                                 if QMessageBox.critical(self,
                                                        self.caption(),
@@ -856,7 +868,7 @@ class SetupForm(SetupForm_base):
         if self.mq.get('fw-download', False):
             try:
                 d = device.Device(self.device_uri)
-            except Error , e:
+            except Error as e:
                 self.FailureUI(self.__tr("<b>Error opening device. Firmware download is Failed.</b><p>%s (%s)." % (e.msg, e.opt)))
             else:
                 if d.downloadFirmware():
@@ -876,9 +888,9 @@ class SetupForm(SetupForm_base):
 
         #if self.ppd_file.startswith("foomatic:"):
         if not os.path.exists(self.ppd_file): # assume foomatic: or some such
-            add_prnt_args = (self.printer_name.encode('utf8'), self.device_uri,self.location, '', self.ppd_file, self.desc)
+            add_prnt_args = (self.printer_name, self.device_uri,self.location, '', self.ppd_file, self.desc)
         else:
-            add_prnt_args = (self.printer_name.encode('utf8'), self.device_uri, self.location, self.ppd_file, '', self.desc)
+            add_prnt_args = (self.printer_name, self.device_uri, self.location, self.ppd_file, '', self.desc)
 
         status, status_str = cups.cups_operation(cups.addPrinter, GUI_MODE, 'qt3', self, *add_prnt_args)
 
@@ -889,12 +901,11 @@ class SetupForm(SetupForm_base):
             self.FailureUI(self.__tr("<b>Printer queue setup failed.</b><p>Error : %s "%status_str))
         else:
             # sending Event to add this device in hp-systray
-            utils.sendEvent(EVENT_CUPS_QUEUES_CHANGED,self.device_uri, self.printer_name)
+            utils.sendEvent(EVENT_CUPS_QUEUES_ADDED,self.device_uri, self.printer_name)
 
         QApplication.restoreOverrideCursor()
         return status
 
-
     def setupFax(self):
         status = cups.IPP_BAD_REQUEST
         QApplication.setOverrideCursor(QApplication.waitCursor)
@@ -915,7 +926,7 @@ class SetupForm(SetupForm_base):
 
                 while True:
                     ppd_dir = sys_conf.get('dirs', 'ppd')
-                    fax_ppd = unicode(QFileDialog.getOpenFileName(ppd_dir,
+                    fax_ppd = to_unicode(QFileDialog.getOpenFileName(ppd_dir,
                         "HP Fax PPD Files (*.ppd *.ppd.gz);;All Files (*)", self,
                         "open file dialog", "Choose the fax PPD file"))
 
@@ -948,7 +959,7 @@ class SetupForm(SetupForm_base):
             self.FailureUI(self.__tr("<b>Fax queue setup failed.</b><p>Error : %s "%status_str))
         else:
             # sending Event to add this device in hp-systray
-            utils.sendEvent(EVENT_CUPS_QUEUES_CHANGED,self.fax_uri, self.fax_name)
+            utils.sendEvent(EVENT_CUPS_QUEUES_ADDED,self.fax_uri, self.fax_name)
 
         QApplication.restoreOverrideCursor()
         return status
@@ -957,7 +968,7 @@ class SetupForm(SetupForm_base):
         if self.print_test_page:
             try:
                 d = device.Device(self.device_uri)
-            except Error, e:
+            except Error as e:
                 self.FailureUI(self.__tr("<b>Device error:</b><p>%s (%s)." % (e.msg, e.opt)))
 
             else:
@@ -971,7 +982,7 @@ class SetupForm(SetupForm_base):
 
                         try:
                             d.printTestPage(self.printer_name)
-                        except Error, e:
+                        except Error as e:
                             if e.opt == ERROR_NO_CUPS_QUEUE_FOUND_FOR_DEVICE:
                                 self.FailureUI(self.__tr("<b>No CUPS queue found for device.</b><p>Please install the printer in CUPS and try again."))
                             else:
@@ -988,7 +999,7 @@ class SetupForm(SetupForm_base):
         QWizard.reject(self)
 
     def FailureUI(self, error_text):
-        log.error(unicode(error_text).replace("<b>", "").replace("</b>", "").replace("<p>", " "))
+        log.error(to_unicode(error_text).replace("<b>", "").replace("</b>", "").replace("<p>", " "))
         QMessageBox.critical(self,
                              self.caption(),
                              error_text,
@@ -1054,10 +1065,10 @@ class PasswordDialog(QDialog):
             self.usernameLineEdit.setPaletteBackgroundColor(QColor("lightgray"))
 
     def getUsername(self):
-        return unicode(self.usernameLineEdit.text())
+        return to_unicode(self.usernameLineEdit.text())
 
     def getPassword(self):
-        return unicode(self.passwordLineEdit.text())
+        return to_unicode(self.passwordLineEdit.text())
 
     def languageChange(self):
         self.setCaption(self.__tr("HP Device Manager - Enter Username/Password"))
